unit FormSavePosition;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, IniFiles, LCLProc;

type

  { TFormSavePosition }

  TFormSavePosition = class
  private
    FForceDirectoryCreate: Boolean;
    FIniFile: TIniFile;
    FTop: Integer;
    FLeft: Integer;
    FHeight: Integer;
    FWidth: Integer;
    FWindowState: TWindowState;

    procedure SetForceDirectoryCreate(const AValue: Boolean);
    procedure SetHeight(const AValue: Integer);
    procedure SetLeft(const AValue: Integer);
    procedure SetTop(const AValue: Integer);
    procedure SetWidth(const AValue: Integer);
    procedure SetWindowState(const AValue: TWindowState);

  public
    constructor Create;
    function LoadFromFile(const Fn: String): Boolean;
    function SaveToFile(const Fn: String): Boolean;
    property Top: Integer read FTop write SetTop;
    property Left: Integer read FLeft write SetLeft;
    property Height: Integer read FHeight write SetHeight;
    property Width: Integer read FWidth write SetWidth;
    property WindowState: TWindowState read FWindowState write SetWindowState;
    property ForceDirectoryCreate: Boolean read FForceDirectoryCreate write SetForceDirectoryCreate default True;
  end;



implementation

const
  WindowStateStrings: Array[TWindowState] of ShortString = ('wsNormal', 'wsMinimized', 'wsMaximized','wsFullScreen');
  { In Lazarus versions < 0.9.31 r31470 compilation of the above fails -> change it into:
    WindowStateStrings: Array[TWindowState] of ShortString = ('wsNormal', 'wsMinimized', 'wsMaximized');
    This is because the definition of TWindowState has changed
  }
  scFormPosition = 'FormPosition';
  idTop = 'Top';
  idLeft = 'Left';
  idWidth = 'Width';
  idHeight = 'Heigth';
  idWindowState = 'WindowState';

{ TFormSavePosition }

function WindowStateToString(const AState: TWindowState): String;
begin
  Result := WindowStateStrings[AState];
end;

function StringToWindowState(AValue: String): TWindowState;
var
  ws: TWindowState;
begin
  Result := wsNormal;
  for ws := Low(TWindowState) to High(TWindowState) do
  begin
    if CompareText(WindowStateStrings[ws], AValue) = 0 then
    begin
      Result := ws;
      //debugln('StringToWindowState: found ',AValue,' -> ',WindowStateStrings[ws]);
      Exit;
    end;
  end;
end;


procedure TFormSavePosition.SetLeft(const AValue: Integer);
begin
  if FLeft=AValue then exit;
  FLeft:=AValue;
end;

procedure TFormSavePosition.SetHeight(const AValue: Integer);
begin
  if FHeight=AValue then exit;
  FHeight:=AValue;
end;

procedure TFormSavePosition.SetForceDirectoryCreate(const AValue: Boolean);
begin
  if FForceDirectoryCreate=AValue then exit;
  FForceDirectoryCreate:=AValue;
end;

procedure TFormSavePosition.SetTop(const AValue: Integer);
begin
  if FTop=AValue then exit;
  FTop:=AValue;
end;

procedure TFormSavePosition.SetWidth(const AValue: Integer);
begin
  if FWidth=AValue then exit;
  FWidth:=AValue;
end;

procedure TFormSavePosition.SetWindowState(const AValue: TWindowState);
begin
  if FWindowState=AValue then exit;
  FWindowState:=AValue;
end;

constructor TFormSavePosition.Create;
begin
  FTop := -1;
  FLeft := -1;
  FWidth := -1;
  FHeight := -1;
  fWindowState := wsNormal;
  FIniFile := Nil;
  FForceDirectoryCreate := True;
end;

function TFormSavePosition.LoadFromFile(const Fn: String): Boolean;
begin
  if not FileExists(Fn) then
  begin
    Result := False;
    Exit;
  end;
  try
    try
      FIniFile := TIniFile.Create(Fn);
      FTop := FIniFile.ReadInteger(scFormPosition, idTop, -1);
      FLeft := FIniFile.ReadInteger(scFormPosition, idLeft, -1);
      FWidth := FIniFile.ReadInteger(scFormPosition, idWidth, -1);
      FHeight := FIniFile.ReadInteger(scFormPosition, idHeight, -1);
      FWindowState := StringToWindowState(FIniFile.ReadString(scFormPosition, idWindowState, WindowStateStrings[wsNormal]));
      Result := true;
    except
      Result := false;
    end;
  finally
    if Assigned(FIniFile) then FInifile.Free;
    FIniFile := nil;
  end;
end;

function TFormSavePosition.SaveToFile(const Fn: String): Boolean;
var
  Dir: String;
begin
  Dir := ExcludeTrailingPathDelimiter(ExtractFileDir(Fn));
  if not DirectoryExists(Dir) then
  begin
    if (not FForceDirectoryCreate) or (not ForceDirectories(Dir)) then
    begin
      Result := False;
      DebugLn('TFormSavePosition.SaveToFile: Directory does not exsist (or could not be created):');
      DebugLn('  -> ',Dir);
      Exit;
    end;
  end;
  try
    try
      FIniFile := TIniFile.Create(Fn);
      FIniFile.CacheUpdates := True;
      FIniFile.WriteInteger(scFormPosition, idTop, FTop);
      FIniFile.WriteInteger(scFormPosition, idLeft, FLeft);
      FIniFile.WriteInteger(scFormPosition, idHeight, FHeight);
      FIniFile.WriteInteger(scFormPosition, idWidth, FWidth);
      FIniFile.WriteString(scFormPosition, idWindowState, WindowStateStrings[FWindowstate]);
      FIniFile.UpdateFile;
      Result := true;
    except
      Result := false;
    end;
  finally
    if Assigned(FIniFile) then
    begin
      Try
        FInifile.Free;
        //Free calls Destroy, Destroy Calls UpdateFile, which tries to write to file,
        //which can raise an exception if the writing fails
        //possible bug in fpc ??
      Except
        Result := False;
      end;
    end;
    FIniFile := nil;
  end;
end;

end.

